"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ClickHouseServer = void 0;
const tslib_1 = require("tslib");
/* istanbul ignore file */
const Docker = require("dockerode");
const faker_1 = require("@faker-js/faker");
const client_1 = require("@clickhouse/client");
const BPromise = require("bluebird");
const docker = new Docker();
/**
 * ClickHouse Server in docker
 * table: users (id Int32, name String, enabled Boolean)
 * rows: 200 rows.
 */
class ClickHouseServer {
    constructor() {
        this.password = '123';
        // https://hub.docker.com/r/clickhouse/clickhouse-server/
        this.image = 'clickhouse/clickhouse-server:23';
        this.port = faker_1.default.datatype.number({ min: 20000, max: 30000 });
        this.host = 'localhost';
        this.user = 'user';
        this.database = 'db';
    }
    prepare() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const pullStream = yield docker.pull(this.image);
            // https://github.com/apocas/dockerode/issues/647
            yield new Promise((res) => docker.modem.followProgress(pullStream, res));
            this.container = yield docker.createContainer({
                name: `vulcan-clickhouse-test-${faker_1.default.datatype.number({ min: 1 })}`,
                Image: this.image,
                ExposedPorts: {
                    '8123/tcp': {},
                },
                Env: [
                    `CLICKHOUSE_DB=${this.database}`,
                    `CLICKHOUSE_USER=${this.user}`,
                    `CLICKHOUSE_PASSWORD=${this.password}`,
                ],
                HostConfig: {
                    PortBindings: {
                        '8123/tcp': [{ HostPort: `${this.port}` }],
                    },
                    // Set unlimited options according to https://hub.docker.com/r/clickhouse/clickhouse-server/
                    // Docs: https://docs.docker.com/engine/api/v1.37/#tag/Container/operation/ContainerCreate
                    Ulimits: [{ Name: 'nofile', Soft: 262144, Hard: 262144 }],
                },
            });
            yield this.container.start({});
            yield this.waitClickHouseReady();
            // Init data
            const client = (0, client_1.createClient)({
                host: `http://${this.host}:${this.port}`,
                username: this.user,
                password: this.password,
                database: this.database,
            });
            yield client.exec({
                query: `CREATE TABLE IF NOT EXISTS products
      (
          serial UInt32, 
          product_id UUID,
          name String,
          price UInt32,
          enabled Boolean
      )
      ENGINE = MergeTree
      PRIMARY KEY (product_id)`,
            });
            yield client.exec({
                query: `INSERT INTO products (*) VALUES
        (1, '6fd1080f-c186-42ba-b32c-10211a8689a2', 'juice', 30, true),
        (2, '17a677ca-8f50-49c5-82d7-bce85031ee09', 'egg', 50, false),
        (3, '9fb20bb4-75ea-46bf-ab00-4898a28283fc', 'milk', 20, true)`,
            });
            for (let i = 4; i <= 200; i++) {
                yield client.exec({
                    query: 'INSERT INTO products (*) VALUES ({serial: Int}, {product_id: UUID}, {name:String}, {price:Float}, {enabled:Boolean})',
                    query_params: {
                        serial: i,
                        product_id: faker_1.default.datatype.uuid(),
                        name: faker_1.default.commerce.product(),
                        price: faker_1.default.commerce.price(100, 300, 0),
                        enabled: faker_1.default.datatype.boolean(),
                    },
                });
            }
        });
    }
    destroy() {
        var _a;
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield ((_a = this.container) === null || _a === void 0 ? void 0 : _a.remove({ force: true }));
        });
    }
    getProfile(name) {
        return {
            name,
            type: 'clickhouse',
            connection: {
                host: `http://${this.host}:${this.port}`,
                username: this.user,
                password: this.password,
                database: this.database,
            },
            allow: '*',
        };
    }
    waitClickHouseReady() {
        var _a;
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const waitConnection = yield ((_a = this.container) === null || _a === void 0 ? void 0 : _a.exec({
                Cmd: [
                    '/bin/bash',
                    '-c',
                    'until grep "<Information> Application: Ready for connections" /var/log/clickhouse-server/clickhouse-server.log; do sleep 5; echo "not ready"; done',
                ],
            }));
            if (!waitConnection)
                return;
            yield waitConnection.start({});
            let wait = 20;
            while (wait--) {
                const { Running, ExitCode } = yield waitConnection.inspect();
                if (!Running && ExitCode === 0)
                    return;
                else if (!Running && ExitCode && ExitCode > 0)
                    throw new Error(`ClickHouse wait commend return exit code ${ExitCode}`);
                yield BPromise.delay(1000);
            }
            throw new Error(`ClickHouse timeout`);
        });
    }
}
exports.ClickHouseServer = ClickHouseServer;
//# sourceMappingURL=clickHouseServer.js.map